@@ -1,10 +1,11 @@
 // PStreams - POSIX Process I/O for C++
 
-//        Copyright (C) 2001 - 2017 Jonathan Wakely
+//        Copyright (C) 2001 - 2020 Jonathan Wakely
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
 //
+// SPDX-License-Identifier: BSL-1.0
 
 #include <cassert>
 
@@ -157,6 +158,9 @@ int main()
 
     std::string str;
 
+    //std::string badcmd = "hgfhdgf";
+    const std::string badcmd = "hgfhdgf 2>/dev/null";
+
     clog << "# Testing basic I/O\n";
 
     {
@@ -437,9 +441,6 @@ int main()
 
     clog << "# Testing behaviour with bad commands" << endl;
 
-    //std::string badcmd = "hgfhdgf";
-    const std::string badcmd = "hgfhdgf 2>/dev/null";
-
     {
         // check is_open() works 
         ipstream is(badcmd);
@@ -477,7 +478,7 @@ int main()
         // does not show failure: print_result(ofail, !ofail.is_open());
         pstreambuf* buf = ofail.rdbuf();
         print_result(ofail, buf->exited());
-        int status = buf->status();
+        int status = ofail.close();
         print_result( ofail,
                 WIFEXITED(status) && WEXITSTATUS(status) == sh_cmd_not_found );
     }
@@ -654,7 +655,7 @@ int main()
         char c;
         std::string s;
         ipstream in("echo 'abc' >&2", pstreambuf::pstderr);
-        print_result(in, getline(in, s));
+        print_result(in, (bool)getline(in, s));
         print_result(in, s == "abc");
         check_fail(in.get(c));
         in.close();
@@ -662,7 +663,7 @@ int main()
         s.erase();
 
         in.open("echo 'abc'", pstreambuf::pstdout);
-        print_result(in, getline(in, s));
+        print_result(in, (bool)getline(in, s));
         print_result(in, s == "abc");
         check_fail(in.get(c));
         in.close();
@@ -670,7 +671,7 @@ int main()
         s.erase();
 
         in.open("echo 'abc' >&2", pstreambuf::pstderr);
-        print_result(in, getline(in, s));
+        print_result(in, (bool)getline(in, s));
         print_result(in, s == "abc");
         check_fail(in.get(c));
         in.close();
@@ -823,8 +824,8 @@ int main()
         class pguard
         {
         public:
-            explicit pguard(ipstream& in, int signal=SIGKILL)
-            : buf_(*in.rdbuf()), signal_(signal) { }
+            explicit pguard(ipstream& istr, int signal=SIGKILL)
+            : buf_(*istr.rdbuf()), signal_(signal) { }
 
             ~pguard() { if (signal_) buf_.kill(signal_); }
 
@@ -849,11 +850,10 @@ int main()
             pguard pg(in, SIGTERM);  // should kill child
             throw 0;
         } catch(...) {
-            in.close();  // should return immediately if child killed
+            int status = in.close();  // should return immediately if child killed
             print_result(in, alarm(0) > 0 && sig_counter == 0);
             sigaction(SIGALRM, &oldact, 0);
 
-            int status = in.rdbuf()->status();
             print_result(in, WIFSIGNALED(status) && WTERMSIG(status)==SIGTERM);
         }
     }
@@ -868,8 +868,7 @@ int main()
         print_result(in, !p);
         print_result(in, in.rdbuf()->error() == EPERM);
         in.rdbuf()->kill(SIGTERM);
-        in.close();
-        int status = in.rdbuf()->status();
+        int status = in.close();
         print_result(in, WIFSIGNALED(status) && WTERMSIG(status)==SIGTERM);
     }
     {
@@ -880,8 +879,7 @@ int main()
         void* p = in.rdbuf()->killpg(SIGKILL);
         print_result(in, p);
         print_result(in, in.rdbuf()->error() == 0);
-        in.close();
-        int status = in.rdbuf()->status();
+        int status = in.close();
         print_result(in, WIFSIGNALED(status) && WTERMSIG(status)==SIGKILL);
     }
 
@@ -892,7 +890,7 @@ int main()
         std::wstring s;
         in >> s;
         wcout << s;
-        print_result(dummy, in);
+        print_result(dummy, (bool)in);
 
         wchar_t wc;
         int count=0, gcount=0;
@@ -908,6 +906,109 @@ int main()
         wcout << L"Read: " << gcount << L" chars." << endl;
     }
 
+#if __cplusplus >= 201103L
+    clog << "# Testing move semantics\n";
+    {
+      ipstream is("echo foo bar baz & echo oof rab zab >&2",
+                  pstreams::pstdout|pstreams::pstderr);
+      is >> str;
+      ipstream is2 = std::move(is);
+      print_result(is2, str == "foo");
+      check_fail(is >> str);
+      is2 >> str;
+      print_result(is2, str == "bar");
+      is2.err();
+      is2 >> str;
+      print_result(is2, str == "oof");
+      is = std::move(is2);
+      check_fail(is2 >> str);
+      is >> str;
+      print_result(is, str == "rab");
+      is.out();
+      is >> str;
+      print_result(is, str == "baz");
+      swap(is, is2);
+      check_fail(is2 >> str);
+      is2.clear();
+      is2.err();
+      is2 >> str;
+      print_result(is2, str == "zab");
+
+      // Check moved-from stream can be re-opened and re-used
+      is.clear();
+      is.open("echo xyzzy >&2", pstreams::pstderr);
+      is >> str;
+      print_result(is, str == "xyzzy");
+      check_fail(is >> str);
+
+      is2.close();
+      swap(is, is2);
+      is.clear();
+      is.open("echo fnord");
+      is >> str;
+      clog << str << '\n';
+      print_result(is, str == "fnord");
+      check_fail(is >> str);
+    }
+
+    {
+      opstream os("awk 'END { if (NR < 3) exit(1) }'");
+      os << "1\n";
+      opstream os2 = std::move(os);
+      check_fail(os << 1);
+      check_pass(os2 << "2\n");
+      os = std::move(os2);
+      check_fail(os2 << 2);
+      check_pass(os << "3\n" << peof);
+      int ex = os.close();
+      print_result(os, ex == 0);
+
+      // Check moved-from stream can be re-opened and re-used
+      os2.clear();
+      os2.open("grep needle >/dev/null");
+      os2 << "hhhaaayyyssstttaaaccckkkneedlehaystack";
+      ex = os2.close();
+      print_result(os2, ex == 0);
+    }
+
+    {
+      pstream ps("sort");
+      ps << "one\n";
+      pstream ps2 = std::move(ps);
+      check_fail(ps << 1);
+      ps2 << "two\n";
+      check_pass(ps2);
+      ps = std::move(ps2);
+      check_fail(ps2 << 1);
+      ps << "three\n";
+      check_pass(ps);
+      swap(ps, ps2);
+      check_fail(ps << 1);
+      ps2 << "four\n";
+      check_pass(ps2);
+      swap(ps, ps2);
+      ps << peof;
+      str.assign(std::istreambuf_iterator<char>(ps), {});
+      check_pass(ps, str == "four\none\nthree\ntwo\n");
+    }
+#endif
 
     return exit_status;
 }
+
+#if __cplusplus >= 201103L
+using std::is_constructible;
+using std::initializer_list;
+static_assert(is_constructible<ipstream, initializer_list<char*>>::value,
+    "initializer-list constructor accepts string-like types");
+static_assert(is_constructible<opstream, initializer_list<char*>>::value,
+    "initializer-list constructor accepts string-like types");
+static_assert(is_constructible<pstream, initializer_list<char*>>::value,
+    "initializer-list constructor accepts string-like types");
+static_assert(!is_constructible<ipstream, initializer_list<int>>::value,
+    "initializer-list constructor doesn't accept bogus types");
+static_assert(!is_constructible<opstream, initializer_list<int>>::value,
+    "initializer-list constructor doesn't accept bogus types");
+static_assert(!is_constructible<pstream, initializer_list<int>>::value,
+    "initializer-list constructor doesn't accept bogus types");
+#endif
